@@ -11,7 +11,7 @@
   by
 
   Travis Oliphant,  oliphant@ee.byu.edu
-  Brigham Young Univeristy
+  Brigham Young University
 
 
 maintainer email:  oliphant.travis@ieee.org
@@ -50,6 +50,11 @@ maintainer email:  oliphant.travis@ieee.org
 #include "sequence.h"
 #include "buffer.h"
 #include "array_assign.h"
+#include "alloc.h"
+#include "mem_overlap.h"
+#include "numpyos.h"
+
+#include "binop_override.h"
 
 /*NUMPY_API
   Compute the size of an array (in number of items)
@@ -94,7 +99,7 @@ PyArray_SetUpdateIfCopyBase(PyArrayObject *arr, PyArrayObject *base)
     }
 
     /*
-     * Any writes to 'arr' will magicaly turn into writes to 'base', so we
+     * Any writes to 'arr' will magically turn into writes to 'base', so we
      * should warn if necessary.
      */
     if (PyArray_FLAGS(base) & NPY_ARRAY_WARN_ON_WRITE) {
@@ -221,8 +226,13 @@ PyArray_CopyObject(PyArrayObject *dest, PyObject *src_object)
         n_new = PyArray_DIMS(dest)[PyArray_NDIM(dest)-1];
         n_old = PyString_Size(src_object);
         if (n_new > n_old) {
-            new_string = (char *)malloc(n_new);
-            memmove(new_string, PyString_AS_STRING(src_object), n_old);
+            new_string = malloc(n_new);
+            if (new_string == NULL) {
+                Py_DECREF(src_object);
+                PyErr_NoMemory();
+                return -1;
+            }
+            memcpy(new_string, PyString_AS_STRING(src_object), n_old);
             memset(new_string + n_old, ' ', n_new - n_old);
             tmp = PyString_FromStringAndSize(new_string, n_new);
             free(new_string);
@@ -414,27 +424,55 @@ array_dealloc(PyArrayObject *self)
              * self already...
              */
         }
-        PyDataMem_FREE(fa->data);
+        npy_free_cache(fa->data, PyArray_NBYTES(self));
     }
 
-    PyDimMem_FREE(fa->dimensions);
+    /* must match allocation in PyArray_NewFromDescr */
+    npy_free_cache_dim(fa->dimensions, 2 * fa->nd);
     Py_DECREF(fa->descr);
     Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
+/*
+ * Extend string. On failure, returns NULL and leaves *strp alone.
+ * XXX we do this in multiple places; time for a string library?
+ */
+static char *
+extend(char **strp, Py_ssize_t n, Py_ssize_t *maxp)
+{
+    char *str = *strp;
+    Py_ssize_t new_cap;
+
+    if (n >= *maxp - 16) {
+        new_cap = *maxp * 2;
+
+        if (new_cap <= *maxp) {     /* overflow */
+            return NULL;
+        }
+        str = PyArray_realloc(*strp, new_cap);
+        if (str != NULL) {
+            *strp = str;
+            *maxp = new_cap;
+        }
+    }
+    return str;
+}
+
 static int
-dump_data(char **string, int *n, int *max_n, char *data, int nd,
+dump_data(char **string, Py_ssize_t *n, Py_ssize_t *max_n, char *data, int nd,
           npy_intp *dimensions, npy_intp *strides, PyArrayObject* self)
 {
     PyArray_Descr *descr=PyArray_DESCR(self);
-    PyObject *op, *sp;
+    PyObject *op = NULL, *sp = NULL;
     char *ostring;
-    npy_intp i, N;
+    npy_intp i, N, ret = 0;
 
-#define CHECK_MEMORY do { if (*n >= *max_n-16) {         \
-        *max_n *= 2;                                     \
-        *string = (char *)PyArray_realloc(*string, *max_n); \
-    }} while (0)
+#define CHECK_MEMORY do {                           \
+        if (extend(string, *n, max_n) == NULL) {    \
+            ret = -1;                               \
+            goto end;                               \
+        }                                           \
+    } while (0)
 
     if (nd == 0) {
         if ((op = descr->f->getitem(data, self)) == NULL) {
@@ -442,17 +480,14 @@ dump_data(char **string, int *n, int *max_n, char *data, int nd,
         }
         sp = PyObject_Repr(op);
         if (sp == NULL) {
-            Py_DECREF(op);
-            return -1;
+            ret = -1;
+            goto end;
         }
         ostring = PyString_AsString(sp);
         N = PyString_Size(sp)*sizeof(char);
         *n += N;
         CHECK_MEMORY;
         memmove(*string + (*n - N), ostring, N);
-        Py_DECREF(sp);
-        Py_DECREF(op);
-        return 0;
     }
     else {
         CHECK_MEMORY;
@@ -475,10 +510,14 @@ dump_data(char **string, int *n, int *max_n, char *data, int nd,
         CHECK_MEMORY;
         (*string)[*n] = ']';
         *n += 1;
-        return 0;
     }
 
 #undef CHECK_MEMORY
+
+end:
+    Py_XDECREF(op);
+    Py_XDECREF(sp);
+    return ret;
 }
 
 /*NUMPY_API
@@ -548,21 +587,13 @@ array_repr_builtin(PyArrayObject *self, int repr)
 {
     PyObject *ret;
     char *string;
-    int n, max_n;
+    /* max_n initial value is arbitrary, dump_data will extend it */
+    Py_ssize_t n = 0, max_n = PyArray_NBYTES(self) * 4 + 7;
 
-    max_n = PyArray_NBYTES(self)*4*sizeof(char) + 7;
-
-    if ((string = (char *)PyArray_malloc(max_n)) == NULL) {
+    if ((string = PyArray_malloc(max_n)) == NULL) {
         return PyErr_NoMemory();
     }
 
-    if (repr) {
-        n = 6;
-        sprintf(string, "array(");
-    }
-    else {
-        n = 0;
-    }
     if (dump_data(&string, &n, &max_n, PyArray_DATA(self),
                   PyArray_NDIM(self), PyArray_DIMS(self),
                   PyArray_STRIDES(self), self) < 0) {
@@ -572,14 +603,15 @@ array_repr_builtin(PyArrayObject *self, int repr)
 
     if (repr) {
         if (PyArray_ISEXTENDED(self)) {
-            char buf[100];
-            PyOS_snprintf(buf, sizeof(buf), "%d", PyArray_DESCR(self)->elsize);
-            sprintf(string+n, ", '%c%s')", PyArray_DESCR(self)->type, buf);
-            ret = PyUString_FromStringAndSize(string, n + 6 + strlen(buf));
+            ret = PyUString_FromFormat("array(%s, '%c%d')",
+                                       string,
+                                       PyArray_DESCR(self)->type,
+                                       PyArray_DESCR(self)->elsize);
         }
         else {
-            sprintf(string+n, ", '%c')", PyArray_DESCR(self)->type);
-            ret = PyUString_FromStringAndSize(string, n+6);
+            ret = PyUString_FromFormat("array(%s, '%c')",
+                                       string,
+                                       PyArray_DESCR(self)->type);
         }
     }
     else {
@@ -704,12 +736,13 @@ array_might_be_written(PyArrayObject *obj)
 {
     const char *msg =
         "Numpy has detected that you (may be) writing to an array returned\n"
-        "by numpy.diagonal or by selecting multiple fields in a record\n"
+        "by numpy.diagonal or by selecting multiple fields in a structured\n"
         "array. This code will likely break in a future numpy release --\n"
         "see numpy.diagonal or arrays.indexing reference docs for details.\n"
         "The quick fix is to make an explicit copy (e.g., do\n"
         "arr.diagonal().copy() or arr[['f0','f1']].copy()).";
     if (PyArray_FLAGS(obj) & NPY_ARRAY_WARN_ON_WRITE) {
+        /* 2012-07-17, 1.7 */
         if (DEPRECATE_FUTUREWARNING(msg) < 0) {
             return -1;
         }
@@ -850,18 +883,13 @@ _mystrncmp(char *s1, char *s2, int len1, int len2)
 
 #define SMALL_STRING 2048
 
-#if defined(isspace)
-#undef isspace
-#define isspace(c)  ((c==' ')||(c=='\t')||(c=='\n')||(c=='\r')||(c=='\v')||(c=='\f'))
-#endif
-
 static void _rstripw(char *s, int n)
 {
     int i;
     for (i = n - 1; i >= 1; i--) { /* Never strip to length 0. */
         int c = s[i];
 
-        if (!c || isspace(c)) {
+        if (!c || NumPyOS_ascii_isspace((int)c)) {
             s[i] = 0;
         }
         else {
@@ -875,7 +903,7 @@ static void _unistripw(npy_ucs4 *s, int n)
     int i;
     for (i = n - 1; i >= 1; i--) { /* Never strip to length 0. */
         npy_ucs4 c = s[i];
-        if (!c || isspace(c)) {
+        if (!c || NumPyOS_ascii_isspace((int)c)) {
             s[i] = 0;
         }
         else {
@@ -1057,7 +1085,7 @@ _strings_richcompare(PyArrayObject *self, PyArrayObject *other, int cmp_op,
 {
     PyArrayObject *result;
     PyArrayMultiIterObject *mit;
-    int val;
+    int val, cast = 0;
 
     /* Cast arrays to a common type */
     if (PyArray_TYPE(self) != PyArray_DESCR(other)->type_num) {
@@ -1069,9 +1097,13 @@ _strings_richcompare(PyArrayObject *self, PyArrayObject *other, int cmp_op,
         Py_INCREF(Py_NotImplemented);
         return Py_NotImplemented;
 #else
+        cast = 1;
+#endif  /* define(NPY_PY3K) */
+    }
+    if (cast || (PyArray_ISNOTSWAPPED(self) != PyArray_ISNOTSWAPPED(other))) {
         PyObject *new;
         if (PyArray_TYPE(self) == NPY_STRING &&
-            PyArray_DESCR(other)->type_num == NPY_UNICODE) {
+                PyArray_DESCR(other)->type_num == NPY_UNICODE) {
             PyArray_Descr* unicode = PyArray_DescrNew(PyArray_DESCR(other));
             unicode->elsize = PyArray_DESCR(self)->elsize << 2;
             new = PyArray_FromAny((PyObject *)self, unicode,
@@ -1082,10 +1114,17 @@ _strings_richcompare(PyArrayObject *self, PyArrayObject *other, int cmp_op,
             Py_INCREF(other);
             self = (PyArrayObject *)new;
         }
-        else if (PyArray_TYPE(self) == NPY_UNICODE &&
-                 PyArray_DESCR(other)->type_num == NPY_STRING) {
+        else if ((PyArray_TYPE(self) == NPY_UNICODE) &&
+                 ((PyArray_DESCR(other)->type_num == NPY_STRING) ||
+                 (PyArray_ISNOTSWAPPED(self) != PyArray_ISNOTSWAPPED(other)))) {
             PyArray_Descr* unicode = PyArray_DescrNew(PyArray_DESCR(self));
-            unicode->elsize = PyArray_DESCR(other)->elsize << 2;
+
+            if (PyArray_DESCR(other)->type_num == NPY_STRING) {
+                unicode->elsize = PyArray_DESCR(other)->elsize << 2;
+            }
+            else {
+                unicode->elsize = PyArray_DESCR(other)->elsize;
+            }
             new = PyArray_FromAny((PyObject *)other, unicode,
                                   0, 0, 0, NULL);
             if (new == NULL) {
@@ -1100,7 +1139,6 @@ _strings_richcompare(PyArrayObject *self, PyArrayObject *other, int cmp_op,
                             "in comparison");
             return NULL;
         }
-#endif
     }
     else {
         Py_INCREF(self);
@@ -1264,28 +1302,50 @@ NPY_NO_EXPORT PyObject *
 array_richcompare(PyArrayObject *self, PyObject *other, int cmp_op)
 {
     PyArrayObject *array_other;
+    PyObject *obj_self = (PyObject *)self;
     PyObject *result = NULL;
 
+    /* Special case for string arrays (which don't and currently can't have
+     * ufunc loops defined, so there's no point in trying).
+     */
+    if (PyArray_ISSTRING(self)) {
+        array_other = (PyArrayObject *)PyArray_FromObject(other,
+                                                          NPY_NOTYPE, 0, 0);
+        if (array_other == NULL) {
+            PyErr_Clear();
+            /* Never mind, carry on, see what happens */
+        }
+        else if (!PyArray_ISSTRING(array_other)) {
+            Py_DECREF(array_other);
+            /* Never mind, carry on, see what happens */
+        }
+        else {
+            result = _strings_richcompare(self, array_other, cmp_op, 0);
+            Py_DECREF(array_other);
+            return result;
+        }
+        /* If we reach this point, it means that we are not comparing
+         * string-to-string. It's possible that this will still work out,
+         * e.g. if the other array is an object array, then both will be cast
+         * to object or something? I don't know how that works actually, but
+         * it does, b/c this works:
+         *   l = ["a", "b"]
+         *   assert np.array(l, dtype="S1") == np.array(l, dtype="O")
+         * So we fall through and see what happens.
+         */
+    }
+
     switch (cmp_op) {
     case Py_LT:
-        result = PyArray_GenericBinaryFunction(self, other,
-                n_ops.less);
+        RICHCMP_GIVE_UP_IF_NEEDED(obj_self, other);
+        result = PyArray_GenericBinaryFunction(self, other, n_ops.less);
         break;
     case Py_LE:
-        result = PyArray_GenericBinaryFunction(self, other,
-                n_ops.less_equal);
+        RICHCMP_GIVE_UP_IF_NEEDED(obj_self, other);
+        result = PyArray_GenericBinaryFunction(self, other, n_ops.less_equal);
         break;
     case Py_EQ:
-        if (other == Py_None) {
-            Py_INCREF(Py_False);
-            return Py_False;
-        }
-        result = PyArray_GenericBinaryFunction(self,
-                (PyObject *)other,
-                n_ops.equal);
-        if (result && result != Py_NotImplemented)
-          break;
-
+        RICHCMP_GIVE_UP_IF_NEEDED(obj_self, other);
         /*
          * The ufunc does not support void/structured types, so these
          * need to be handled specifically. Only a few cases are supported.
@@ -1294,55 +1354,71 @@ array_richcompare(PyArrayObject *self, PyObject *other, int cmp_op)
         if (PyArray_TYPE(self) == NPY_VOID) {
             int _res;
 
-            array_other = (PyArrayObject *)PyArray_FromAny(other, NULL, 0, 0, 0,
-                                                           NULL);
+            array_other = (PyArrayObject *)PyArray_FROM_O(other);
             /*
              * If not successful, indicate that the items cannot be compared
              * this way.
              */
             if (array_other == NULL) {
+                /* 2015-05-07, 1.10 */
                 PyErr_Clear();
+                if (DEPRECATE(
+                        "elementwise == comparison failed and returning scalar "
+                        "instead; this will raise an error in the future.") < 0) {
+                    return NULL;
+                }
                 Py_INCREF(Py_NotImplemented);
                 return Py_NotImplemented;
             }
 
-            _res = PyObject_RichCompareBool
-                ((PyObject *)PyArray_DESCR(self),
-                 (PyObject *)PyArray_DESCR(array_other),
-                 Py_EQ);
-            if (_res < 0) {
-                Py_DECREF(result);
+            _res = PyArray_CanCastTypeTo(PyArray_DESCR(self),
+                                         PyArray_DESCR(array_other),
+                                         NPY_EQUIV_CASTING);
+            if (_res == 0) {
+                /* 2015-05-07, 1.10 */
                 Py_DECREF(array_other);
-                return NULL;
+                if (DEPRECATE_FUTUREWARNING(
+                        "elementwise == comparison failed and returning scalar "
+                        "instead; this will raise an error or perform "
+                        "elementwise comparison in the future.") < 0) {
+                    return NULL;
+                }
+                Py_INCREF(Py_False);
+                return Py_False;
             }
-            if (_res) {
-                Py_DECREF(result);
+            else {
                 result = _void_compare(self, array_other, cmp_op);
             }
             Py_DECREF(array_other);
             return result;
         }
+
+        result = PyArray_GenericBinaryFunction(self,
+                (PyObject *)other,
+                n_ops.equal);
         /*
          * If the comparison results in NULL, then the
          * two array objects can not be compared together;
          * indicate that
          */
         if (result == NULL) {
+            /*
+             * Comparisons should raise errors when element-wise comparison
+             * is not possible.
+             */
+            /* 2015-05-14, 1.10 */
             PyErr_Clear();
+            if (DEPRECATE("elementwise == comparison failed; "
+                          "this will raise an error in the future.") < 0) {
+                return NULL;
+            }
+
             Py_INCREF(Py_NotImplemented);
             return Py_NotImplemented;
         }
         break;
     case Py_NE:
-        if (other == Py_None) {
-            Py_INCREF(Py_True);
-            return Py_True;
-        }
-        result = PyArray_GenericBinaryFunction(self, (PyObject *)other,
-                n_ops.not_equal);
-        if (result && result != Py_NotImplemented)
-          break;
-
+        RICHCMP_GIVE_UP_IF_NEEDED(obj_self, other);
         /*
          * The ufunc does not support void/structured types, so these
          * need to be handled specifically. Only a few cases are supported.
@@ -1351,46 +1427,70 @@ array_richcompare(PyArrayObject *self, PyObject *other, int cmp_op)
         if (PyArray_TYPE(self) == NPY_VOID) {
             int _res;
 
-            array_other = (PyArrayObject *)PyArray_FromAny(other, NULL, 0, 0, 0,
-                                                           NULL);
+            array_other = (PyArrayObject *)PyArray_FROM_O(other);
             /*
              * If not successful, indicate that the items cannot be compared
              * this way.
             */
             if (array_other == NULL) {
+                /* 2015-05-07, 1.10 */
                 PyErr_Clear();
+                if (DEPRECATE(
+                        "elementwise != comparison failed and returning scalar "
+                        "instead; this will raise an error in the future.") < 0) {
+                    return NULL;
+                }
                 Py_INCREF(Py_NotImplemented);
                 return Py_NotImplemented;
             }
 
-            _res = PyObject_RichCompareBool(
-                    (PyObject *)PyArray_DESCR(self),
-                    (PyObject *)PyArray_DESCR(array_other),
-                    Py_EQ);
-            if (_res < 0) {
-                Py_DECREF(result);
+            _res = PyArray_CanCastTypeTo(PyArray_DESCR(self),
+                                         PyArray_DESCR(array_other),
+                                         NPY_EQUIV_CASTING);
+            if (_res == 0) {
+                /* 2015-05-07, 1.10 */
                 Py_DECREF(array_other);
-                return NULL;
+                if (DEPRECATE_FUTUREWARNING(
+                        "elementwise != comparison failed and returning scalar "
+                        "instead; this will raise an error or perform "
+                        "elementwise comparison in the future.") < 0) {
+                    return NULL;
+                }
+                Py_INCREF(Py_True);
+                return Py_True;
             }
-            if (_res) {
-                Py_DECREF(result);
+            else {
                 result = _void_compare(self, array_other, cmp_op);
                 Py_DECREF(array_other);
             }
             return result;
         }
 
+        result = PyArray_GenericBinaryFunction(self, (PyObject *)other,
+                n_ops.not_equal);
         if (result == NULL) {
+            /*
+             * Comparisons should raise errors when element-wise comparison
+             * is not possible.
+             */
+            /* 2015-05-14, 1.10 */
             PyErr_Clear();
+            if (DEPRECATE("elementwise != comparison failed; "
+                          "this will raise an error in the future.") < 0) {
+                return NULL;
+            }
+
             Py_INCREF(Py_NotImplemented);
             return Py_NotImplemented;
         }
         break;
     case Py_GT:
+        RICHCMP_GIVE_UP_IF_NEEDED(obj_self, other);
         result = PyArray_GenericBinaryFunction(self, other,
                 n_ops.greater);
         break;
     case Py_GE:
+        RICHCMP_GIVE_UP_IF_NEEDED(obj_self, other);
         result = PyArray_GenericBinaryFunction(self, other,
                 n_ops.greater_equal);
         break;
@@ -1398,24 +1498,6 @@ array_richcompare(PyArrayObject *self, PyObject *other, int cmp_op)
         result = Py_NotImplemented;
         Py_INCREF(result);
     }
-    if (result == Py_NotImplemented) {
-        /* Try to handle string comparisons */
-        if (PyArray_TYPE(self) == NPY_OBJECT) {
-            return result;
-        }
-        array_other = (PyArrayObject *)PyArray_FromObject(other,
-                                                          NPY_NOTYPE, 0, 0);
-        if (array_other == NULL) {
-            PyErr_Clear();
-            return result;
-        }
-        if (PyArray_ISSTRING(self) && PyArray_ISSTRING(array_other)) {
-            Py_DECREF(result);
-            result = _strings_richcompare(self, (PyArrayObject *)
-                                          array_other, cmp_op, 0);
-        }
-        Py_DECREF(array_other);
-    }
     return result;
 }
 
@@ -1511,7 +1593,7 @@ array_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
      * strides, and swapped info For now, let's just use this to create an
      * empty, contiguous array of a specific type and shape.
      */
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&|O&O&LO&O&",
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&|O&O&LO&O&:ndarray",
                                      kwlist, PyArray_IntpConverter,
                                      &dims,
                                      PyArray_DescrConverter,
@@ -1533,11 +1615,6 @@ array_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
     }
 
     itemsize = descr->elsize;
-    if (itemsize == 0) {
-        PyErr_SetString(PyExc_ValueError,
-                        "data-type with unspecified variable length");
-        goto fail;
-    }
 
     if (strides.ptr != NULL) {
         npy_intp nb, off;
@@ -1571,10 +1648,11 @@ array_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
 
     if (buffer.ptr == NULL) {
         ret = (PyArrayObject *)
-            PyArray_NewFromDescr(subtype, descr,
-                                 (int)dims.len,
-                                 dims.ptr,
-                                 strides.ptr, NULL, is_f_order, NULL);
+            PyArray_NewFromDescr_int(subtype, descr,
+                                     (int)dims.len,
+                                     dims.ptr,
+                                     strides.ptr, NULL, is_f_order, NULL,
+                                     0, 1);
         if (ret == NULL) {
             descr = NULL;
             goto fail;
@@ -1607,11 +1685,11 @@ array_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
             buffer.flags |= NPY_ARRAY_F_CONTIGUOUS;
         }
         ret = (PyArrayObject *)\
-            PyArray_NewFromDescr(subtype, descr,
-                                 dims.len, dims.ptr,
-                                 strides.ptr,
-                                 offset + (char *)buffer.ptr,
-                                 buffer.flags, NULL);
+            PyArray_NewFromDescr_int(subtype, descr,
+                                     dims.len, dims.ptr,
+                                     strides.ptr,
+                                     offset + (char *)buffer.ptr,
+                                     buffer.flags, NULL, 0, 1);
         if (ret == NULL) {
             descr = NULL;
             goto fail;
@@ -1626,19 +1704,13 @@ array_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
     }
 
     PyDimMem_FREE(dims.ptr);
-    if (strides.ptr) {
-        PyDimMem_FREE(strides.ptr);
-    }
+    PyDimMem_FREE(strides.ptr);
     return (PyObject *)ret;
 
  fail:
     Py_XDECREF(descr);
-    if (dims.ptr) {
-        PyDimMem_FREE(dims.ptr);
-    }
-    if (strides.ptr) {
-        PyDimMem_FREE(strides.ptr);
-    }
+    PyDimMem_FREE(dims.ptr);
+    PyDimMem_FREE(strides.ptr);
     return NULL;
 }
 
@@ -1657,13 +1729,19 @@ array_iter(PyArrayObject *arr)
 static PyObject *
 array_alloc(PyTypeObject *type, Py_ssize_t NPY_UNUSED(nitems))
 {
-    PyObject *obj;
     /* nitems will always be 0 */
-    obj = (PyObject *)PyArray_malloc(type->tp_basicsize);
+    PyObject *obj = PyObject_Malloc(type->tp_basicsize);
     PyObject_Init(obj, type);
     return obj;
 }
 
+static void
+array_free(PyObject * v)
+{
+    /* avoid same deallocator as PyBaseObject, see gentype_free */
+    PyObject_Free(v);
+}
+
 
 NPY_NO_EXPORT PyTypeObject PyArray_Type = {
 #if defined(NPY_PY3K)
@@ -1689,6 +1767,10 @@ NPY_NO_EXPORT PyTypeObject PyArray_Type = {
     &array_as_number,                           /* tp_as_number */
     &array_as_sequence,                         /* tp_as_sequence */
     &array_as_mapping,                          /* tp_as_mapping */
+    /*
+     * The tp_hash slot will be set PyObject_HashNotImplemented when the
+     * module is loaded.
+     */
     (hashfunc)0,                                /* tp_hash */
     (ternaryfunc)0,                             /* tp_call */
     (reprfunc)array_str,                        /* tp_str */
@@ -1700,7 +1782,7 @@ NPY_NO_EXPORT PyTypeObject PyArray_Type = {
      | Py_TPFLAGS_CHECKTYPES
      | Py_TPFLAGS_HAVE_NEWBUFFER
 #endif
-     | Py_TPFLAGS_BASETYPE),                  /* tp_flags */
+     | Py_TPFLAGS_BASETYPE),                    /* tp_flags */
     0,                                          /* tp_doc */
 
     (traverseproc)0,                            /* tp_traverse */
@@ -1718,9 +1800,9 @@ NPY_NO_EXPORT PyTypeObject PyArray_Type = {
     0,                                          /* tp_descr_set */
     0,                                          /* tp_dictoffset */
     (initproc)0,                                /* tp_init */
-    array_alloc,                                /* tp_alloc */
+    (allocfunc)array_alloc,                     /* tp_alloc */
     (newfunc)array_new,                         /* tp_new */
-    0,                                          /* tp_free */
+    (freefunc)array_free,                       /* tp_free */
     0,                                          /* tp_is_gc */
     0,                                          /* tp_bases */
     0,                                          /* tp_mro */
