@@ -830,28 +830,24 @@ _GenericBinaryOutFunction(PyArrayObject *m1, PyObject *m2, PyArrayObject *out,
         return PyObject_CallFunction(op, "OO", m1, m2);
     }
     else {
-        PyObject *args, *kw, *ret;
+        PyObject *args, *ret;
+        static PyObject *kw = NULL;
+
+        if (kw == NULL) {
+            kw = Py_BuildValue("{s:s}", "casting", "unsafe");
+            if (kw == NULL) {
+                return NULL;
+            }
+        }
 
         args = Py_BuildValue("OOO", m1, m2, out);
         if (args == NULL) {
             return NULL;
         }
-        kw = PyDict_New();
-        if (kw == NULL) {
-            Py_DECREF(args);
-            return NULL;
-        }
-        if (PyDict_SetItemString(kw, "casting",
-                        PyUString_FromString("unsafe")) < 0) {
-            Py_DECREF(args);
-            Py_DECREF(kw);
-            return NULL;
-        }
 
         ret = PyObject_Call(op, args, kw);
 
         Py_DECREF(args);
-        Py_DECREF(kw);
 
         return ret;
     }
@@ -1177,7 +1173,8 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
 NPY_NO_EXPORT PyObject *
 PyArray_Conjugate(PyArrayObject *self, PyArrayObject *out)
 {
-    if (PyArray_ISCOMPLEX(self) || PyArray_ISOBJECT(self)) {
+    if (PyArray_ISCOMPLEX(self) || PyArray_ISOBJECT(self) ||
+            PyArray_ISUSERDEF(self)) {
         if (out == NULL) {
             return PyArray_GenericUnaryFunction(self,
                                                 n_ops.conjugate);
@@ -1190,6 +1187,14 @@ PyArray_Conjugate(PyArrayObject *self, PyArrayObject *out)
     }
     else {
         PyArrayObject *ret;
+        if (!PyArray_ISNUMBER(self)) {
+            /* 2017-05-04, 1.13 */
+            if (DEPRECATE("attempting to conjugate non-numeric dtype; this "
+                          "will error in the future to match the behavior of "
+                          "np.conjugate") < 0) {
+                return NULL;
+            }
+        }
         if (out) {
             if (PyArray_AssignArray(out, self,
                         NULL, NPY_DEFAULT_ASSIGN_CASTING) < 0) {
