@@ -5,75 +5,129 @@ cpuinfo
 Copyright 2002 Pearu Peterson all rights reserved,
 Pearu Peterson <pearu@cens.ioc.ee>
 Permission to use, modify, and distribute this software is given under the
-terms of the SciPy (BSD style) license.  See LICENSE.txt that came with
+terms of the NumPy (BSD style) license.  See LICENSE.txt that came with
 this distribution for specifics.
 
-Note:  This should be merged into proc at some point.  Perhaps proc should
-be returning classes like this instead of using dictionaries.
-
 NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
-$Revision: 1.1 $
-$Date: 2005/04/09 19:29:34 $
 Pearu Peterson
-"""
 
-__version__ = "$Id: cpuinfo.py,v 1.1 2005/04/09 19:29:34 pearu Exp $"
+"""
+from __future__ import division, absolute_import, print_function
 
 __all__ = ['cpu']
 
-import sys,string,re,types
+import sys, re, types
+import os
 
-class cpuinfo_base:
+if sys.version_info[0] >= 3:
+    from subprocess import getstatusoutput
+else:
+    from commands import getstatusoutput
+
+import warnings
+import platform
+
+from numpy.distutils.compat import get_exception
+
+def getoutput(cmd, successful_status=(0,), stacklevel=1):
+    try:
+        status, output = getstatusoutput(cmd)
+    except EnvironmentError:
+        e = get_exception()
+        warnings.warn(str(e), UserWarning, stacklevel=stacklevel)
+        return False, output
+    if os.WIFEXITED(status) and os.WEXITSTATUS(status) in successful_status:
+        return True, output
+    return False, output
+
+def command_info(successful_status=(0,), stacklevel=1, **kw):
+    info = {}
+    for key in kw:
+        ok, output = getoutput(kw[key], successful_status=successful_status,
+                               stacklevel=stacklevel+1)
+        if ok:
+            info[key] = output.strip()
+    return info
+
+def command_by_line(cmd, successful_status=(0,), stacklevel=1):
+    ok, output = getoutput(cmd, successful_status=successful_status,
+                           stacklevel=stacklevel+1)
+    if not ok:
+        return
+    for line in output.splitlines():
+        yield line.strip()
+
+def key_value_from_command(cmd, sep, successful_status=(0,),
+                           stacklevel=1):
+    d = {}
+    for line in command_by_line(cmd, successful_status=successful_status,
+                                stacklevel=stacklevel+1):
+        l = [s.strip() for s in line.split(sep, 1)]
+        if len(l) == 2:
+            d[l[0]] = l[1]
+    return d
+
+class CPUInfoBase(object):
     """Holds CPU information and provides methods for requiring
     the availability of various CPU features.
     """
 
-    def _try_call(self,func):
+    def _try_call(self, func):
         try:
             return func()
         except:
             pass
 
-    def __getattr__(self,name):
-        if name[0]!='_':
-            if hasattr(self,'_'+name):
-                attr = getattr(self,'_'+name)
-                if type(attr) is types.MethodType:
+    def __getattr__(self, name):
+        if not name.startswith('_'):
+            if hasattr(self, '_'+name):
+                attr = getattr(self, '_'+name)
+                if isinstance(attr, types.MethodType):
                     return lambda func=self._try_call,attr=attr : func(attr)
             else:
                 return lambda : None
-        raise AttributeError,name
+        raise AttributeError(name)
 
     def _getNCPUs(self):
         return 1
 
-    def _is_32bit(self):
-        return not self.is_64bit()
+    def __get_nbits(self):
+        abits = platform.architecture()[0]
+        nbits = re.compile(r'(\d+)bit').search(abits).group(1)
+        return nbits
 
-class linux_cpuinfo(cpuinfo_base):
+    def _is_32bit(self):
+        return self.__get_nbits() == '32'
+
+    def _is_64bit(self):
+        return self.__get_nbits() == '64'
+
+class LinuxCPUInfo(CPUInfoBase):
 
     info = None
 
     def __init__(self):
         if self.info is not None:
             return
-        info = []
+        info = [ {} ]
+        ok, output = getoutput('uname -m')
+        if ok:
+            info[0]['uname_m'] = output.strip()
         try:
-            for line in open('/proc/cpuinfo').readlines():
-                name_value = map(string.strip,string.split(line,':',1))
-                if len(name_value)!=2:
+            fo = open('/proc/cpuinfo')
+        except EnvironmentError:
+            e = get_exception()
+            warnings.warn(str(e), UserWarning, stacklevel=2)
+        else:
+            for line in fo:
+                name_value = [s.strip() for s in line.split(':', 1)]
+                if len(name_value) != 2:
                     continue
-                name,value = name_value
-                if not info or info[-1].has_key(name): # next processor
+                name, value = name_value
+                if not info or name in info[-1]: # next processor
                     info.append({})
                 info[-1][name] = value
-            import commands
-            status,output = commands.getstatusoutput('uname -m')
-            if not status:
-                if not info: info.append({})
-                info[-1]['uname_m'] = string.strip(output)
-        except:
-            print sys.exc_value,'(ignoring)'
+            fo.close()
         self.__class__.info = info
 
     def _not_impl(self): pass
@@ -90,15 +144,18 @@ class linux_cpuinfo(cpuinfo_base):
         return self._is_AMD() and self.info[0]['model'] == '3'
 
     def _is_AthlonK6(self):
-        return re.match(r'.*?AMD-K6',self.info[0]['model name']) is not None
+        return re.match(r'.*?AMD-K6', self.info[0]['model name']) is not None
 
     def _is_AthlonK7(self):
-        return re.match(r'.*?AMD-K7',self.info[0]['model name']) is not None
+        return re.match(r'.*?AMD-K7', self.info[0]['model name']) is not None
 
     def _is_AthlonMP(self):
         return re.match(r'.*?Athlon\(tm\) MP\b',
                         self.info[0]['model name']) is not None
 
+    def _is_AMD64(self):
+        return self.is_AMD() and self.info[0]['family'] == '15'
+
     def _is_Athlon64(self):
         return re.match(r'.*?Athlon\(tm\) 64\b',
                         self.info[0]['model name']) is not None
@@ -185,7 +242,16 @@ class linux_cpuinfo(cpuinfo_base):
         return self.is_PentiumIV() and self.has_sse3()
 
     def _is_Nocona(self):
-        return self.is_PentiumIV() and self.is_64bit()
+        return self.is_Intel() \
+               and (self.info[0]['cpu family'] == '6' \
+                    or self.info[0]['cpu family'] == '15' ) \
+               and (self.has_sse3() and not self.has_ssse3())\
+               and re.match(r'.*?\blm\b', self.info[0]['flags']) is not None
+
+    def _is_Core2(self):
+        return self.is_64bit() and self.is_Intel() and \
+               re.match(r'.*?Core\(TM\)2\b', \
+                        self.info[0]['model name']) is not None
 
     def _is_Itanium(self):
         return re.match(r'.*?Itanium\b',
@@ -193,7 +259,7 @@ class linux_cpuinfo(cpuinfo_base):
 
     def _is_XEON(self):
         return re.match(r'.*?XEON\b',
-                        self.info[0]['model name'],re.IGNORECASE) is not None
+                        self.info[0]['model name'], re.IGNORECASE) is not None
 
     _is_Xeon = _is_XEON
 
@@ -212,73 +278,46 @@ class linux_cpuinfo(cpuinfo_base):
         return self.info[0]['f00f_bug']=='yes'
 
     def _has_mmx(self):
-        return re.match(r'.*?\bmmx\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bmmx\b', self.info[0]['flags']) is not None
 
     def _has_sse(self):
-        return re.match(r'.*?\bsse\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bsse\b', self.info[0]['flags']) is not None
 
     def _has_sse2(self):
-        return re.match(r'.*?\bsse2\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bsse2\b', self.info[0]['flags']) is not None
 
     def _has_sse3(self):
-        return re.match(r'.*?\bsse3\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bpni\b', self.info[0]['flags']) is not None
+
+    def _has_ssse3(self):
+        return re.match(r'.*?\bssse3\b', self.info[0]['flags']) is not None
 
     def _has_3dnow(self):
-        return re.match(r'.*?\b3dnow\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\b3dnow\b', self.info[0]['flags']) is not None
 
     def _has_3dnowext(self):
-        return re.match(r'.*?\b3dnowext\b',self.info[0]['flags']) is not None
-
-    def _is_64bit(self):
-        if self.is_Alpha():
-            return True
-        if self.info[0].get('clflush size','')=='64':
-            return True
-        if self.info[0].get('uname_m','')=='x86_64':
-            return True
-        if self.info[0].get('arch','')=='IA-64':
-            return True
-        return False
-
-    def _is_32bit(self):
-        return not self.is_64bit()
-
-class irix_cpuinfo(cpuinfo_base):
+        return re.match(r'.*?\b3dnowext\b', self.info[0]['flags']) is not None
 
+class IRIXCPUInfo(CPUInfoBase):
     info = None
 
     def __init__(self):
         if self.info is not None:
             return
-        info = []
-        try:
-            import commands
-            status,output = commands.getstatusoutput('sysconf')
-            if status not in [0,256]:
-                return
-            for line in output.split('\n'):
-                name_value = map(string.strip,string.split(line,' ',1))
-                if len(name_value)!=2:
-                    continue
-                name,value = name_value
-                if not info:
-                    info.append({})
-                info[-1][name] = value
-        except:
-            print sys.exc_value,'(ignoring)'
+        info = key_value_from_command('sysconf', sep=' ',
+                                      successful_status=(0, 1))
         self.__class__.info = info
 
-        #print info
     def _not_impl(self): pass
 
     def _is_singleCPU(self):
-        return self.info[0].get('NUM_PROCESSORS') == '1'
+        return self.info.get('NUM_PROCESSORS') == '1'
 
     def _getNCPUs(self):
-        return int(self.info[0].get('NUM_PROCESSORS'))
+        return int(self.info.get('NUM_PROCESSORS', 1))
 
-    def __cputype(self,n):
-        return self.info[0].get('PROCESSORS').split()[0].lower() == 'r%s' % (n)
+    def __cputype(self, n):
+        return self.info.get('PROCESSORS').split()[0].lower() == 'r%s' % (n)
     def _is_r2000(self): return self.__cputype(2000)
     def _is_r3000(self): return self.__cputype(3000)
     def _is_r3900(self): return self.__cputype(3900)
@@ -296,10 +335,10 @@ class irix_cpuinfo(cpuinfo_base):
     def _is_rorion(self): return self.__cputype('orion')
 
     def get_ip(self):
-        try: return self.info[0].get('MACHINE')
+        try: return self.info.get('MACHINE')
         except: pass
-    def __machine(self,n):
-        return self.info[0].get('MACHINE').lower() == 'ip%s' % (n)
+    def __machine(self, n):
+        return self.info.get('MACHINE').lower() == 'ip%s' % (n)
     def _is_IP19(self): return self.__machine(19)
     def _is_IP20(self): return self.__machine(20)
     def _is_IP21(self): return self.__machine(21)
@@ -316,53 +355,33 @@ class irix_cpuinfo(cpuinfo_base):
     def _is_IP32_5k(self): return self.__machine(32) and self._is_r5000()
     def _is_IP32_10k(self): return self.__machine(32) and self._is_r10000()
 
-class darwin_cpuinfo(cpuinfo_base):
 
+class DarwinCPUInfo(CPUInfoBase):
     info = None
 
     def __init__(self):
         if self.info is not None:
             return
-        info = []
-        try:
-            import commands
-            status,output = commands.getstatusoutput('arch')
-            if not status:
-                if not info: info.append({})
-                info[-1]['arch'] = string.strip(output)
-            status,output = commands.getstatusoutput('machine')
-            if not status:
-                if not info: info.append({})
-                info[-1]['machine'] = string.strip(output)
-            status,output = commands.getstatusoutput('sysctl hw')
-            if not status:
-                if not info: info.append({})
-                d = {}
-                for l in string.split(output,'\n'):
-                    l = map(string.strip,string.split(l, '='))
-                    if len(l)==2:
-                        d[l[0]]=l[1]
-                info[-1]['sysctl_hw'] = d
-        except:
-            print sys.exc_value,'(ignoring)'
+        info = command_info(arch='arch',
+                            machine='machine')
+        info['sysctl_hw'] = key_value_from_command('sysctl hw', sep='=')
         self.__class__.info = info
 
     def _not_impl(self): pass
 
     def _getNCPUs(self):
-        try: return int(self.info[0]['sysctl_hw']['hw.ncpu'])
-        except: return 1
+        return int(self.info['sysctl_hw'].get('hw.ncpu', 1))
 
     def _is_Power_Macintosh(self):
-        return self.info[0]['sysctl_hw']['hw.machine']=='Power Macintosh'
+        return self.info['sysctl_hw']['hw.machine']=='Power Macintosh'
 
     def _is_i386(self):
-        return self.info[0]['arch']=='i386'
+        return self.info['arch']=='i386'
     def _is_ppc(self):
-        return self.info[0]['arch']=='ppc'
+        return self.info['arch']=='ppc'
 
-    def __machine(self,n):
-        return self.info[0]['machine'] == 'ppc%s'%n
+    def __machine(self, n):
+        return self.info['machine'] == 'ppc%s'%n
     def _is_ppc601(self): return self.__machine(601)
     def _is_ppc602(self): return self.__machine(602)
     def _is_ppc603(self): return self.__machine(603)
@@ -382,120 +401,85 @@ class darwin_cpuinfo(cpuinfo_base):
     def _is_ppc823(self): return self.__machine(823)
     def _is_ppc860(self): return self.__machine(860)
 
-class sunos_cpuinfo(cpuinfo_base):
+
+class SunOSCPUInfo(CPUInfoBase):
 
     info = None
 
     def __init__(self):
         if self.info is not None:
             return
-        info = []
-        try:
-            import commands
-            status,output = commands.getstatusoutput('arch')
-            if not status:
-                if not info: info.append({})
-                info[-1]['arch'] = string.strip(output)
-            status,output = commands.getstatusoutput('mach')
-            if not status:
-                if not info: info.append({})
-                info[-1]['mach'] = string.strip(output)
-            status,output = commands.getstatusoutput('uname -i')
-            if not status:
-                if not info: info.append({})
-                info[-1]['uname_i'] = string.strip(output)
-            status,output = commands.getstatusoutput('uname -X')
-            if not status:
-                if not info: info.append({})
-                d = {}
-                for l in string.split(output,'\n'):
-                    l = map(string.strip,string.split(l, '='))
-                    if len(l)==2:
-                        d[l[0]]=l[1]
-                info[-1]['uname_X'] = d
-            status,output = commands.getstatusoutput('isainfo -b')
-            if not status:
-                if not info: info.append({})
-                info[-1]['isainfo_b'] = string.strip(output)
-            status,output = commands.getstatusoutput('isainfo -n')
-            if not status:
-                if not info: info.append({})
-                info[-1]['isainfo_n'] = string.strip(output)
-            status,output = commands.getstatusoutput('psrinfo -v 0')
-            if not status:
-                if not info: info.append({})
-                for l in string.split(output,'\n'):
-                    m = re.match(r'\s*The (?P<p>[\w\d]+) processor operates at',l)
-                    if m:
-                        info[-1]['processor'] = m.group('p')
-                        break
-        except:
-            print sys.exc_value,'(ignoring)'
+        info = command_info(arch='arch',
+                            mach='mach',
+                            uname_i='uname_i',
+                            isainfo_b='isainfo -b',
+                            isainfo_n='isainfo -n',
+                            )
+        info['uname_X'] = key_value_from_command('uname -X', sep='=')
+        for line in command_by_line('psrinfo -v 0'):
+            m = re.match(r'\s*The (?P<p>[\w\d]+) processor operates at', line)
+            if m:
+                info['processor'] = m.group('p')
+                break
         self.__class__.info = info
 
     def _not_impl(self): pass
 
-    def _is_32bit(self):
-        return self.info[0]['isainfo_b']=='32'
-    def _is_64bit(self):
-        return self.info[0]['isainfo_b']=='64'
-
     def _is_i386(self):
-        return self.info[0]['isainfo_n']=='i386'
+        return self.info['isainfo_n']=='i386'
     def _is_sparc(self):
-        return self.info[0]['isainfo_n']=='sparc'
+        return self.info['isainfo_n']=='sparc'
     def _is_sparcv9(self):
-        return self.info[0]['isainfo_n']=='sparcv9'
+        return self.info['isainfo_n']=='sparcv9'
 
     def _getNCPUs(self):
-        try: return int(self.info[0]['uname_X']['NumCPU'])
-        except: return 1
+        return int(self.info['uname_X'].get('NumCPU', 1))
 
     def _is_sun4(self):
-        return self.info[0]['arch']=='sun4'
+        return self.info['arch']=='sun4'
 
     def _is_SUNW(self):
-        return re.match(r'SUNW',self.info[0]['uname_i']) is not None
+        return re.match(r'SUNW', self.info['uname_i']) is not None
     def _is_sparcstation5(self):
-        return re.match(r'.*SPARCstation-5',self.info[0]['uname_i']) is not None
+        return re.match(r'.*SPARCstation-5', self.info['uname_i']) is not None
     def _is_ultra1(self):
-        return re.match(r'.*Ultra-1',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-1', self.info['uname_i']) is not None
     def _is_ultra250(self):
-        return re.match(r'.*Ultra-250',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-250', self.info['uname_i']) is not None
     def _is_ultra2(self):
-        return re.match(r'.*Ultra-2',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-2', self.info['uname_i']) is not None
     def _is_ultra30(self):
-        return re.match(r'.*Ultra-30',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-30', self.info['uname_i']) is not None
     def _is_ultra4(self):
-        return re.match(r'.*Ultra-4',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-4', self.info['uname_i']) is not None
     def _is_ultra5_10(self):
-        return re.match(r'.*Ultra-5_10',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-5_10', self.info['uname_i']) is not None
     def _is_ultra5(self):
-        return re.match(r'.*Ultra-5',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-5', self.info['uname_i']) is not None
     def _is_ultra60(self):
-        return re.match(r'.*Ultra-60',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-60', self.info['uname_i']) is not None
     def _is_ultra80(self):
-        return re.match(r'.*Ultra-80',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-80', self.info['uname_i']) is not None
     def _is_ultraenterprice(self):
-        return re.match(r'.*Ultra-Enterprise',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-Enterprise', self.info['uname_i']) is not None
     def _is_ultraenterprice10k(self):
-        return re.match(r'.*Ultra-Enterprise-10000',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra-Enterprise-10000', self.info['uname_i']) is not None
     def _is_sunfire(self):
-        return re.match(r'.*Sun-Fire',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Sun-Fire', self.info['uname_i']) is not None
     def _is_ultra(self):
-        return re.match(r'.*Ultra',self.info[0]['uname_i']) is not None
+        return re.match(r'.*Ultra', self.info['uname_i']) is not None
 
     def _is_cpusparcv7(self):
-        return self.info[0]['processor']=='sparcv7'
+        return self.info['processor']=='sparcv7'
     def _is_cpusparcv8(self):
-        return self.info[0]['processor']=='sparcv8'
+        return self.info['processor']=='sparcv8'
     def _is_cpusparcv9(self):
-        return self.info[0]['processor']=='sparcv9'
+        return self.info['processor']=='sparcv9'
 
-class win32_cpuinfo(cpuinfo_base):
+class Win32CPUInfo(CPUInfoBase):
 
     info = None
-    pkey = "HARDWARE\\DESCRIPTION\\System\\CentralProcessor"
+    pkey = r"HARDWARE\DESCRIPTION\System\CentralProcessor"
     # XXX: what does the value of
     #   HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0
     # mean?
@@ -506,27 +490,29 @@ class win32_cpuinfo(cpuinfo_base):
         info = []
         try:
             #XXX: Bad style to use so long `try:...except:...`. Fix it!
-            import _winreg
-            pkey = "HARDWARE\\DESCRIPTION\\System\\CentralProcessor"
-            prgx = re.compile(r"family\s+(?P<FML>\d+)\s+model\s+(?P<MDL>\d+)"\
-                              "\s+stepping\s+(?P<STP>\d+)",re.IGNORECASE)
-            chnd=_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE,pkey)
+            if sys.version_info[0] >= 3:
+                import winreg
+            else:
+                import _winreg as winreg
+
+            prgx = re.compile(r"family\s+(?P<FML>\d+)\s+model\s+(?P<MDL>\d+)"
+                              r"\s+stepping\s+(?P<STP>\d+)", re.IGNORECASE)
+            chnd=winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, self.pkey)
             pnum=0
-            while 1:
+            while True:
                 try:
-                    proc=_winreg.EnumKey(chnd,pnum)
-                except _winreg.error:
+                    proc=winreg.EnumKey(chnd, pnum)
+                except winreg.error:
                     break
                 else:
                     pnum+=1
-                    print proc
                     info.append({"Processor":proc})
-                    phnd=_winreg.OpenKey(chnd,proc)
+                    phnd=winreg.OpenKey(chnd, proc)
                     pidx=0
                     while True:
                         try:
-                            name,value,vtpe=_winreg.EnumValue(phnd,pidx)
-                        except _winreg.error:
+                            name, value, vtpe=winreg.EnumValue(phnd, pidx)
+                        except winreg.error:
                             break
                         else:
                             pidx=pidx+1
@@ -538,7 +524,7 @@ class win32_cpuinfo(cpuinfo_base):
                                     info[-1]["Model"]=int(srch.group("MDL"))
                                     info[-1]["Stepping"]=int(srch.group("STP"))
         except:
-            print sys.exc_value,'(ignoring)'
+            print(sys.exc_info()[1], '(ignoring)')
         self.__class__.info = info
 
     def _not_impl(self): pass
@@ -556,11 +542,11 @@ class win32_cpuinfo(cpuinfo_base):
 
     def _is_AMDK5(self):
         return self.is_AMD() and self.info[0]['Family']==5 \
-               and self.info[0]['Model'] in [0,1,2,3]
+               and self.info[0]['Model'] in [0, 1, 2, 3]
 
     def _is_AMDK6(self):
         return self.is_AMD() and self.info[0]['Family']==5 \
-               and self.info[0]['Model'] in [6,7]
+               and self.info[0]['Model'] in [6, 7]
 
     def _is_AMDK6_2(self):
         return self.is_AMD() and self.info[0]['Family']==5 \
@@ -570,16 +556,15 @@ class win32_cpuinfo(cpuinfo_base):
         return self.is_AMD() and self.info[0]['Family']==5 \
                and self.info[0]['Model']==9
 
-    def _is_Athlon(self):
-        return self.is_AMD() and self.info[0]['Family']==6
+    def _is_AMDK7(self):
+        return self.is_AMD() and self.info[0]['Family'] == 6
 
-    def _is_Athlon64(self):
-        return self.is_AMD() and self.info[0]['Family']==15 \
-               and self.info[0]['Model']==4
+    # To reliably distinguish between the different types of AMD64 chips
+    # (Athlon64, Operton, Athlon64 X2, Semperon, Turion 64, etc.) would
+    # require looking at the 'brand' from cpuid
 
-    def _is_Opteron(self):
-        return self.is_AMD() and self.info[0]['Family']==15 \
-               and self.info[0]['Model']==5
+    def _is_AMD64(self):
+        return self.is_AMD() and self.info[0]['Family'] == 15
 
     # Intel
 
@@ -611,15 +596,23 @@ class win32_cpuinfo(cpuinfo_base):
 
     def _is_PentiumII(self):
         return self.is_Intel() and self.info[0]['Family']==6 \
-               and self.info[0]['Model'] in [3,5,6]
+               and self.info[0]['Model'] in [3, 5, 6]
 
     def _is_PentiumIII(self):
         return self.is_Intel() and self.info[0]['Family']==6 \
-               and self.info[0]['Model'] in [7,8,9,10,11]
+               and self.info[0]['Model'] in [7, 8, 9, 10, 11]
 
     def _is_PentiumIV(self):
         return self.is_Intel() and self.info[0]['Family']==15
 
+    def _is_PentiumM(self):
+        return self.is_Intel() and self.info[0]['Family'] == 6 \
+               and self.info[0]['Model'] in [9, 13, 14]
+
+    def _is_Core2(self):
+        return self.is_Intel() and self.info[0]['Family'] == 6 \
+               and self.info[0]['Model'] in [15, 16, 17]
+
     # Varia
 
     def _is_singleCPU(self):
@@ -631,61 +624,70 @@ class win32_cpuinfo(cpuinfo_base):
     def _has_mmx(self):
         if self.is_Intel():
             return (self.info[0]['Family']==5 and self.info[0]['Model']==4) \
-                   or (self.info[0]['Family'] in [6,15])
+                   or (self.info[0]['Family'] in [6, 15])
         elif self.is_AMD():
-            return self.info[0]['Family'] in [5,6,15]
+            return self.info[0]['Family'] in [5, 6, 15]
+        else:
+            return False
 
     def _has_sse(self):
         if self.is_Intel():
             return (self.info[0]['Family']==6 and \
-                    self.info[0]['Model'] in [7,8,9,10,11]) \
+                    self.info[0]['Model'] in [7, 8, 9, 10, 11]) \
                     or self.info[0]['Family']==15
         elif self.is_AMD():
             return (self.info[0]['Family']==6 and \
-                    self.info[0]['Model'] in [6,7,8,10]) \
+                    self.info[0]['Model'] in [6, 7, 8, 10]) \
                     or self.info[0]['Family']==15
+        else:
+            return False
 
     def _has_sse2(self):
-        return self.info[0]['Family']==15
+        if self.is_Intel():
+            return self.is_Pentium4() or self.is_PentiumM() \
+                   or self.is_Core2()
+        elif self.is_AMD():
+            return self.is_AMD64()
+        else:
+            return False
 
     def _has_3dnow(self):
-        # XXX: does only AMD have 3dnow??
-        return self.is_AMD() and self.info[0]['Family'] in [5,6,15]
+        return self.is_AMD() and self.info[0]['Family'] in [5, 6, 15]
 
     def _has_3dnowext(self):
-        return self.is_AMD() and self.info[0]['Family'] in [6,15]
+        return self.is_AMD() and self.info[0]['Family'] in [6, 15]
 
-if sys.platform[:5] == 'linux': # variations: linux2,linux-i386 (any others?)
-    cpuinfo = linux_cpuinfo
-elif sys.platform[:4] == 'irix':
-    cpuinfo = irix_cpuinfo
+if sys.platform.startswith('linux'): # variations: linux2,linux-i386 (any others?)
+    cpuinfo = LinuxCPUInfo
+elif sys.platform.startswith('irix'):
+    cpuinfo = IRIXCPUInfo
 elif sys.platform == 'darwin':
-    cpuinfo = darwin_cpuinfo
-elif sys.platform[:5] == 'sunos':
-    cpuinfo = sunos_cpuinfo
-elif sys.platform[:5] == 'win32':
-    cpuinfo = win32_cpuinfo
-elif sys.platform[:6] == 'cygwin':
-    cpuinfo = linux_cpuinfo
+    cpuinfo = DarwinCPUInfo
+elif sys.platform.startswith('sunos'):
+    cpuinfo = SunOSCPUInfo
+elif sys.platform.startswith('win32'):
+    cpuinfo = Win32CPUInfo
+elif sys.platform.startswith('cygwin'):
+    cpuinfo = LinuxCPUInfo
 #XXX: other OS's. Eg. use _winreg on Win32. Or os.uname on unices.
 else:
-    cpuinfo = cpuinfo_base
+    cpuinfo = CPUInfoBase
 
 cpu = cpuinfo()
 
-if __name__ == "__main__":
-
-    cpu.is_blaa()
-    cpu.is_Intel()
-    cpu.is_Alpha()
-
-    print 'CPU information:',
-    for name in dir(cpuinfo):
-        if name[0]=='_' and name[1]!='_':
-            r = getattr(cpu,name[1:])()
-            if r:
-                if r!=1:
-                    print '%s=%s' %(name[1:],r),
-                else:
-                    print name[1:],
-    print
+#if __name__ == "__main__":
+#
+#    cpu.is_blaa()
+#    cpu.is_Intel()
+#    cpu.is_Alpha()
+#
+#    print('CPU information:'),
+#    for name in dir(cpuinfo):
+#        if name[0]=='_' and name[1]!='_':
+#            r = getattr(cpu,name[1:])()
+#            if r:
+#                if r!=1:
+#                    print('%s=%s' %(name[1:],r))
+#                else:
+#                    print(name[1:]),
+#    print()
