@@ -20,10 +20,9 @@
 
 int xerbla_(char *srname, integer *info)
 {
-        const char* format = "On entry to %.*s" \
+        static const char format[] = "On entry to %.*s" \
                 " parameter number %d had an illegal value";
-        char buf[57 + 6 + 4]; /* 57 for strlen(format),
-                                 6 for name, 4 for param. num. */
+        char buf[sizeof(format) + 6 + 4];   /* 6 for name, 4 for param. num. */
 
         int len = 0; /* length of subroutine name*/
 #ifdef WITH_THREAD
