@@ -7,47 +7,48 @@ import numpy as np
 import numpy.polynomial.legendre as leg
 from numpy.polynomial.polynomial import polyval
 from numpy.testing import (
-        TestCase, assert_almost_equal, assert_raises,
-        assert_equal, assert_, run_module_suite)
+    TestCase, assert_almost_equal, assert_raises,
+    assert_equal, assert_, run_module_suite)
 
-L0 = np.array([ 1])
-L1 = np.array([ 0,  1])
-L2 = np.array([-1,  0,    3])/2
-L3 = np.array([ 0, -3,    0,    5])/2
-L4 = np.array([ 3,  0,  -30,    0,  35])/8
-L5 = np.array([ 0, 15,    0,  -70,   0,   63])/8
-L6 = np.array([-5,  0,  105,    0, -315,    0,   231])/16
-L7 = np.array([ 0, -35,    0,  315,   0, -693,     0,   429])/16
-L8 = np.array([35,  0, -1260,    0, 6930,    0, -12012,     0, 6435])/128
-L9 = np.array([ 0, 315,    0, -4620,   0, 18018,     0, -25740,   0, 12155])/128
+L0 = np.array([1])
+L1 = np.array([0, 1])
+L2 = np.array([-1, 0, 3])/2
+L3 = np.array([0, -3, 0, 5])/2
+L4 = np.array([3, 0, -30, 0, 35])/8
+L5 = np.array([0, 15, 0, -70, 0, 63])/8
+L6 = np.array([-5, 0, 105, 0, -315, 0, 231])/16
+L7 = np.array([0, -35, 0, 315, 0, -693, 0, 429])/16
+L8 = np.array([35, 0, -1260, 0, 6930, 0, -12012, 0, 6435])/128
+L9 = np.array([0, 315, 0, -4620, 0, 18018, 0, -25740, 0, 12155])/128
 
 Llist = [L0, L1, L2, L3, L4, L5, L6, L7, L8, L9]
 
-def trim(x) :
+
+def trim(x):
     return leg.legtrim(x, tol=1e-6)
 
 
-class TestConstants(TestCase) :
+class TestConstants(TestCase):
 
-    def test_legdomain(self) :
+    def test_legdomain(self):
         assert_equal(leg.legdomain, [-1, 1])
 
-    def test_legzero(self) :
+    def test_legzero(self):
         assert_equal(leg.legzero, [0])
 
-    def test_legone(self) :
+    def test_legone(self):
         assert_equal(leg.legone, [1])
 
-    def test_legx(self) :
+    def test_legx(self):
         assert_equal(leg.legx, [0, 1])
 
 
-class TestArithmetic(TestCase) :
+class TestArithmetic(TestCase):
     x = np.linspace(-1, 1, 100)
 
-    def test_legadd(self) :
-        for i in range(5) :
-            for j in range(5) :
+    def test_legadd(self):
+        for i in range(5):
+            for j in range(5):
                 msg = "At i=%d, j=%d" % (i, j)
                 tgt = np.zeros(max(i, j) + 1)
                 tgt[i] += 1
@@ -55,9 +56,9 @@ class TestArithmetic(TestCase) :
                 res = leg.legadd([0]*i + [1], [0]*j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
-    def test_legsub(self) :
-        for i in range(5) :
-            for j in range(5) :
+    def test_legsub(self):
+        for i in range(5):
+            for j in range(5):
                 msg = "At i=%d, j=%d" % (i, j)
                 tgt = np.zeros(max(i, j) + 1)
                 tgt[i] += 1
@@ -74,12 +75,12 @@ class TestArithmetic(TestCase) :
             tgt = [0]*(i - 1) + [i/tmp, 0, (i + 1)/tmp]
             assert_equal(leg.legmulx(ser), tgt)
 
-    def test_legmul(self) :
+    def test_legmul(self):
         # check values of result
-        for i in range(5) :
+        for i in range(5):
             pol1 = [0]*i + [1]
             val1 = leg.legval(self.x, pol1)
-            for j in range(5) :
+            for j in range(5):
                 msg = "At i=%d, j=%d" % (i, j)
                 pol2 = [0]*j + [1]
                 val2 = leg.legval(self.x, pol2)
@@ -88,9 +89,9 @@ class TestArithmetic(TestCase) :
                 assert_(len(pol3) == i + j + 1, msg)
                 assert_almost_equal(val3, val1*val2, err_msg=msg)
 
-    def test_legdiv(self) :
-        for i in range(5) :
-            for j in range(5) :
+    def test_legdiv(self):
+        for i in range(5):
+            for j in range(5):
                 msg = "At i=%d, j=%d" % (i, j)
                 ci = [0]*i + [1]
                 cj = [0]*j + [1]
@@ -100,7 +101,7 @@ class TestArithmetic(TestCase) :
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
 
-class TestEvaluation(TestCase) :
+class TestEvaluation(TestCase):
     # coefficients of 1 + 2*x + 3*x**2
     c1d = np.array([2., 2., 2.])
     c2d = np.einsum('i,j->ij', c1d, c1d)
@@ -110,22 +111,21 @@ class TestEvaluation(TestCase) :
     x = np.random.random((3, 5))*2 - 1
     y = polyval(x, [1., 2., 3.])
 
-    def test_legval(self) :
+    def test_legval(self):
         #check empty input
         assert_equal(leg.legval([], [1]).size, 0)
 
         #check normal input)
         x = np.linspace(-1, 1)
         y = [polyval(x, c) for c in Llist]
-        for i in range(10) :
+        for i in range(10):
             msg = "At i=%d" % i
-            ser = np.zeros
             tgt = y[i]
             res = leg.legval(x, [0]*i + [1])
             assert_almost_equal(res, tgt, err_msg=msg)
 
         #check that shape is preserved
-        for i in range(3) :
+        for i in range(3):
             dims = [2]*i
             x = np.zeros(dims)
             assert_equal(leg.legval(x, [1]).shape, dims)
@@ -195,9 +195,9 @@ class TestEvaluation(TestCase) :
         assert_(res.shape == (2, 3)*3)
 
 
-class TestIntegral(TestCase) :
+class TestIntegral(TestCase):
 
-    def test_legint(self) :
+    def test_legint(self):
         # check exceptions
         assert_raises(ValueError, leg.legint, [0], .5)
         assert_raises(ValueError, leg.legint, [0], -1)
@@ -210,7 +210,7 @@ class TestIntegral(TestCase) :
             assert_almost_equal(res, [0, 1])
 
         # check single integration with integration constant
-        for i in range(5) :
+        for i in range(5):
             scl = i + 1
             pol = [0]*i + [1]
             tgt = [i] + [0]*i + [1/scl]
@@ -220,7 +220,7 @@ class TestIntegral(TestCase) :
             assert_almost_equal(trim(res), trim(tgt))
 
         # check single integration with integration constant and lbnd
-        for i in range(5) :
+        for i in range(5):
             scl = i + 1
             pol = [0]*i + [1]
             legpol = leg.poly2leg(pol)
@@ -228,7 +228,7 @@ class TestIntegral(TestCase) :
             assert_almost_equal(leg.legval(-1, legint), i)
 
         # check single integration with integration constant and scaling
-        for i in range(5) :
+        for i in range(5):
             scl = i + 1
             pol = [0]*i + [1]
             tgt = [i] + [0]*i + [2/scl]
@@ -238,41 +238,41 @@ class TestIntegral(TestCase) :
             assert_almost_equal(trim(res), trim(tgt))
 
         # check multiple integrations with default k
-        for i in range(5) :
-            for j in range(2, 5) :
+        for i in range(5):
+            for j in range(2, 5):
                 pol = [0]*i + [1]
                 tgt = pol[:]
-                for k in range(j) :
+                for k in range(j):
                     tgt = leg.legint(tgt, m=1)
                 res = leg.legint(pol, m=j)
                 assert_almost_equal(trim(res), trim(tgt))
 
         # check multiple integrations with defined k
-        for i in range(5) :
-            for j in range(2, 5) :
+        for i in range(5):
+            for j in range(2, 5):
                 pol = [0]*i + [1]
                 tgt = pol[:]
-                for k in range(j) :
+                for k in range(j):
                     tgt = leg.legint(tgt, m=1, k=[k])
                 res = leg.legint(pol, m=j, k=list(range(j)))
                 assert_almost_equal(trim(res), trim(tgt))
 
         # check multiple integrations with lbnd
-        for i in range(5) :
-            for j in range(2, 5) :
+        for i in range(5):
+            for j in range(2, 5):
                 pol = [0]*i + [1]
                 tgt = pol[:]
-                for k in range(j) :
+                for k in range(j):
                     tgt = leg.legint(tgt, m=1, k=[k], lbnd=-1)
                 res = leg.legint(pol, m=j, k=list(range(j)), lbnd=-1)
                 assert_almost_equal(trim(res), trim(tgt))
 
         # check multiple integrations with scaling
-        for i in range(5) :
-            for j in range(2, 5) :
+        for i in range(5):
+            for j in range(2, 5):
                 pol = [0]*i + [1]
                 tgt = pol[:]
-                for k in range(j) :
+                for k in range(j):
                     tgt = leg.legint(tgt, m=1, k=[k], scl=2)
                 res = leg.legint(pol, m=j, k=list(range(j)), scl=2)
                 assert_almost_equal(trim(res), trim(tgt))
@@ -294,29 +294,29 @@ class TestIntegral(TestCase) :
         assert_almost_equal(res, tgt)
 
 
-class TestDerivative(TestCase) :
+class TestDerivative(TestCase):
 
-    def test_legder(self) :
+    def test_legder(self):
         # check exceptions
         assert_raises(ValueError, leg.legder, [0], .5)
         assert_raises(ValueError, leg.legder, [0], -1)
 
-        # check that zeroth deriviative does nothing
-        for i in range(5) :
+        # check that zeroth derivative does nothing
+        for i in range(5):
             tgt = [0]*i + [1]
             res = leg.legder(tgt, m=0)
             assert_equal(trim(res), trim(tgt))
 
         # check that derivation is the inverse of integration
-        for i in range(5) :
-            for j in range(2, 5) :
+        for i in range(5):
+            for j in range(2, 5):
                 tgt = [0]*i + [1]
                 res = leg.legder(leg.legint(tgt, m=j), m=j)
                 assert_almost_equal(trim(res), trim(tgt))
 
         # check derivation with scaling
-        for i in range(5) :
-            for j in range(2, 5) :
+        for i in range(5):
+            for j in range(2, 5):
                 tgt = [0]*i + [1]
                 res = leg.legder(leg.legint(tgt, m=j, scl=2), m=j, scl=.5)
                 assert_almost_equal(trim(res), trim(tgt))
@@ -338,13 +338,12 @@ class TestVander(TestCase):
     # some random values in [-1, 1)
     x = np.random.random((3, 5))*2 - 1
 
-
-    def test_legvander(self) :
+    def test_legvander(self):
         # check for 1d x
         x = np.arange(3)
         v = leg.legvander(x, 3)
         assert_(v.shape == (3, 4))
-        for i in range(4) :
+        for i in range(4):
             coef = [0]*i + [1]
             assert_almost_equal(v[..., i], leg.legval(x, coef))
 
@@ -352,11 +351,11 @@ class TestVander(TestCase):
         x = np.array([[1, 2], [3, 4], [5, 6]])
         v = leg.legvander(x, 3)
         assert_(v.shape == (3, 2, 4))
-        for i in range(4) :
+        for i in range(4):
             coef = [0]*i + [1]
             assert_almost_equal(v[..., i], leg.legval(x, coef))
 
-    def test_legvander2d(self) :
+    def test_legvander2d(self):
         # also tests polyval2d for non-square coefficient array
         x1, x2, x3 = self.x
         c = np.random.random((2, 3))
@@ -369,7 +368,7 @@ class TestVander(TestCase):
         van = leg.legvander2d([x1], [x2], [1, 2])
         assert_(van.shape == (1, 5, 6))
 
-    def test_legvander3d(self) :
+    def test_legvander3d(self):
         # also tests polyval3d for non-square coefficient array
         x1, x2, x3 = self.x
         c = np.random.random((2, 3, 4))
@@ -385,19 +384,25 @@ class TestVander(TestCase):
 
 class TestFitting(TestCase):
 
-    def test_legfit(self) :
-        def f(x) :
+    def test_legfit(self):
+        def f(x):
             return x*(x - 1)*(x - 2)
 
+        def f2(x):
+            return x**4 + x**2 + 1
+
         # Test exceptions
-        assert_raises(ValueError, leg.legfit, [1],    [1],     -1)
-        assert_raises(TypeError,  leg.legfit, [[1]],  [1],      0)
-        assert_raises(TypeError,  leg.legfit, [],     [1],      0)
-        assert_raises(TypeError,  leg.legfit, [1],    [[[1]]],  0)
-        assert_raises(TypeError,  leg.legfit, [1, 2], [1],      0)
-        assert_raises(TypeError,  leg.legfit, [1],    [1, 2],   0)
-        assert_raises(TypeError,  leg.legfit, [1],    [1],   0, w=[[1]])
-        assert_raises(TypeError,  leg.legfit, [1],    [1],   0, w=[1, 1])
+        assert_raises(ValueError, leg.legfit, [1], [1], -1)
+        assert_raises(TypeError, leg.legfit, [[1]], [1], 0)
+        assert_raises(TypeError, leg.legfit, [], [1], 0)
+        assert_raises(TypeError, leg.legfit, [1], [[[1]]], 0)
+        assert_raises(TypeError, leg.legfit, [1, 2], [1], 0)
+        assert_raises(TypeError, leg.legfit, [1], [1, 2], 0)
+        assert_raises(TypeError, leg.legfit, [1], [1], 0, w=[[1]])
+        assert_raises(TypeError, leg.legfit, [1], [1], 0, w=[1, 1])
+        assert_raises(ValueError, leg.legfit, [1], [1], [-1,])
+        assert_raises(ValueError, leg.legfit, [1], [1], [2, -1, 6])
+        assert_raises(TypeError, leg.legfit, [1], [1], [])
 
         # Test fit
         x = np.linspace(0, 2)
@@ -406,13 +411,25 @@ class TestFitting(TestCase):
         coef3 = leg.legfit(x, y, 3)
         assert_equal(len(coef3), 4)
         assert_almost_equal(leg.legval(x, coef3), y)
+        coef3 = leg.legfit(x, y, [0, 1, 2, 3])
+        assert_equal(len(coef3), 4)
+        assert_almost_equal(leg.legval(x, coef3), y)
         #
         coef4 = leg.legfit(x, y, 4)
         assert_equal(len(coef4), 5)
         assert_almost_equal(leg.legval(x, coef4), y)
+        coef4 = leg.legfit(x, y, [0, 1, 2, 3, 4])
+        assert_equal(len(coef4), 5)
+        assert_almost_equal(leg.legval(x, coef4), y)
+        # check things still work if deg is not in strict increasing
+        coef4 = leg.legfit(x, y, [2, 3, 4, 1, 0])
+        assert_equal(len(coef4), 5)
+        assert_almost_equal(leg.legval(x, coef4), y)
         #
         coef2d = leg.legfit(x, np.array([y, y]).T, 3)
         assert_almost_equal(coef2d, np.array([coef3, coef3]).T)
+        coef2d = leg.legfit(x, np.array([y, y]).T, [0, 1, 2, 3])
+        assert_almost_equal(coef2d, np.array([coef3, coef3]).T)
         # test weighting
         w = np.zeros_like(x)
         yw = y.copy()
@@ -420,13 +437,26 @@ class TestFitting(TestCase):
         y[0::2] = 0
         wcoef3 = leg.legfit(x, yw, 3, w=w)
         assert_almost_equal(wcoef3, coef3)
+        wcoef3 = leg.legfit(x, yw, [0, 1, 2, 3], w=w)
+        assert_almost_equal(wcoef3, coef3)
         #
         wcoef2d = leg.legfit(x, np.array([yw, yw]).T, 3, w=w)
         assert_almost_equal(wcoef2d, np.array([coef3, coef3]).T)
+        wcoef2d = leg.legfit(x, np.array([yw, yw]).T, [0, 1, 2, 3], w=w)
+        assert_almost_equal(wcoef2d, np.array([coef3, coef3]).T)
         # test scaling with complex values x points whose square
         # is zero when summed.
         x = [1, 1j, -1, -1j]
         assert_almost_equal(leg.legfit(x, x, 1), [0, 1])
+        assert_almost_equal(leg.legfit(x, x, [0, 1]), [0, 1])
+        # test fitting only even Legendre polynomials
+        x = np.linspace(-1, 1)
+        y = f2(x)
+        coef1 = leg.legfit(x, y, 4)
+        assert_almost_equal(leg.legval(x, coef1), y)
+        coef2 = leg.legfit(x, y, [0, 2, 4])
+        assert_almost_equal(leg.legval(x, coef2), y)
+        assert_almost_equal(coef1, coef2)
 
 
 class TestCompanion(TestCase):
@@ -463,12 +493,12 @@ class TestGauss(TestCase):
         assert_almost_equal(w.sum(), tgt)
 
 
-class TestMisc(TestCase) :
+class TestMisc(TestCase):
 
-    def test_legfromroots(self) :
+    def test_legfromroots(self):
         res = leg.legfromroots([])
         assert_almost_equal(trim(res), [1])
-        for i in range(1, 5) :
+        for i in range(1, 5):
             roots = np.cos(np.linspace(-np.pi, 0, 2*i + 1)[1::2])
             pol = leg.legfromroots(roots)
             res = leg.legval(roots, pol)
@@ -477,15 +507,15 @@ class TestMisc(TestCase) :
             assert_almost_equal(leg.leg2poly(pol)[-1], 1)
             assert_almost_equal(res, tgt)
 
-    def test_legroots(self) :
+    def test_legroots(self):
         assert_almost_equal(leg.legroots([1]), [])
         assert_almost_equal(leg.legroots([1, 2]), [-.5])
-        for i in range(2, 5) :
+        for i in range(2, 5):
             tgt = np.linspace(-1, 1, i)
             res = leg.legroots(leg.legfromroots(tgt))
             assert_almost_equal(trim(res), trim(tgt))
 
-    def test_legtrim(self) :
+    def test_legtrim(self):
         coef = [2, -1, 1, 0]
 
         # Test exceptions
@@ -496,15 +526,15 @@ class TestMisc(TestCase) :
         assert_equal(leg.legtrim(coef, 1), coef[:-3])
         assert_equal(leg.legtrim(coef, 2), [0])
 
-    def test_legline(self) :
+    def test_legline(self):
         assert_equal(leg.legline(3, 4), [3, 4])
 
-    def test_leg2poly(self) :
-        for i in range(10) :
+    def test_leg2poly(self):
+        for i in range(10):
             assert_almost_equal(leg.leg2poly([0]*i + [1]), Llist[i])
 
-    def test_poly2leg(self) :
-        for i in range(10) :
+    def test_poly2leg(self):
+        for i in range(10):
             assert_almost_equal(leg.poly2leg(Llist[i]), [0]*i + [1])
 
     def test_weight(self):
